#charset "us-ascii"
#include <adv3.h>
#include <en_us.h>

/* Ladder, by Eric Eve
 */

class DownLadder : NameAsOther, Platform
    construct(ladder_) {
        targetObj = ladder_;
        initializeVocabWith(targetObj.vocabWords);
        inherited;
    }
    getFacets = [targetObj]
    mainMoveInto(newDest) {
        if(newDest is in (nil, targetObj.upperEndLoc))
            inherited(newDest);
        else
            targetObj.mainMoveInto(newDest);
    }
    dobjFor(ClimbDown) asDobjFor(StandOn)
    actionDobjStandOn 
    { 
       gActor.mainMoveInto(targetObj.location); 
       targetObj.actionDobjStandOn; 
    }
    specialDesc = "\^<<aName>> leads down to <<targetObj.location.destName>>. "
    remoteSpecialDesc(pov)
    {
      if(!pov.canSee(targetObj))
       "\^<<aName>> leads down to <<targetObj.location.destName>> from <<location.destName>>. ";
    }
    dobjFor(Topple)
    {
      verify() { }
      check() { targetObj.checkDobjTopple; }
      action() { targetObj.actionDobjTopple; }      
    }
    dobjFor(Push) asDobjFor(Topple)
    dobjFor(Pull) asDobjFor(Take)
    checkDobjTake() { targetObj.checkDobjTake; }
;

class Ladder : Platform
    isEquivalent = true
    upperEndLoc = nil
    lowerEndLoc = nil
    up = (upperEndLoc)
    down = (lowerEndLoc)
    out = nil    
    wallObj = nil
    dobjFor(ClimbUp) asDobjFor(Climb)
    dobjFor(Climb) {
        verify() {
            if(upperEndLoc != nil)
                logicalRank(140, 'ladder');
            else
                illogicalNow('{The dobj/he} is not climbable right now.' );
        }
        action() {
            if(gActor.isIn(self))
                replaceAction(Up);
            else
                replaceAction(StandOn, self);
        }
    }
    
    dobjFor(ClimbDown) {
        verify() {
          if ( !gActor.isIn(self) ) 
             illogicalNow( '{You/he} can\'t climb down {the dobj/him} from here. ');
        }
        action() {
           replaceAction(Down);
        }
    }
    
    dobjFor(PutAgainst) {
        preCond = [objHeld]
        verify() { logicalRank (120, 'ladder'); }
        check() {
          if(gActor.isGhost) {
            "{You/he} {are} too insubstantial to manipulate {the dobj/him}. ";
            exit;
          }          
        }
        action() {
            "{You/he} put{s} {the dobj/him} against {the iobj/him}.\n";
            upperEndLoc = gIobj.upperEndLoc;
            lowerEndLoc = gIobj.lowerEndLoc;
            wallObj = gIobj;
            wallObj.curLadder = self;
            mainMoveInto(lowerEndLoc);
            otherEnd.mainMoveInto(upperEndLoc);
        }
    }

// fix this: it should only reroute for wall, not everything:

    dobjFor(PutOn) maybeRemapTo((gIobj && gIobj.ofKind(Wall)), PutAgainst, self, IndirectObject)

    // this hack protects from actors standing on toppled ladders.
    dobjFor(StandOn) maybeRemapTo(!upperEndLoc, Take, self)

    dobjFor(DropDown)
  {
    verify() { logicalRank (120, 'ladder'); }
    action()
    {
      "{You/he} lower{s} {the dobj/him} down to {the iobj/him}. ";
      upperEndLoc = gIobj.upperEndLoc;
      lowerEndLoc = gIobj.lowerEndLoc;
      mainMoveInto(lowerEndLoc);
      otherEnd.mainMoveInto(upperEndLoc);
      wallObj = gIobj.lowerWall;
      wallObj.curLadder = self;
    }
  } 
    otherEnd = perInstance (new DownLadder(self))
    getFacets = [otherEnd]
    mainMoveInto(newDest) {        
        if(newDest != lowerEndLoc) {
            otherEnd.mainMoveInto(nil);
            upperEndLoc = nil;
            lowerEndLoc = nil;
            if (wallObj)
                wallObj.curLadder = nil;
            wallObj = nil;
        }
        inherited(newDest);
    }
    specialDesc = "\^<<aName>> leans against <<wallObj.theName>>. "
    useSpecialDesc { return (wallObj != nil); }
    remoteSpecialDesc(pov)
    {
      if(pov.isIn(upperEndLoc))
        "";
      else
        "\^<<aName>> leans against <<wallObj.theName>> in <<wallObj.location.destName>>. ";
    }
    dobjFor(Topple)
    {
      verify() {
        if(upperEndLoc == nil)
          illogicalNow('{The dobj/he} is not leaning against anything. ');
        if(gActor.isIn(self))
          illogicalNow('{You/he} can\'t push {the dobj/him} over while {you\'re/he\'s}
            on it. ');
      }
      check() {
        if(getWeight - weight > gActor.weight)
        {
          "\^<<gActor.theName>> pushes with all his might, but {the dobj/he} is too heavy. ";
          exit;
        }
        if(gActor.isGhost)
        {
          "{You/he} {are} too insubstantial to manage that. ";
          exit;
        }  
      }
      action()
      {
        local loc = lowerEndLoc;
        "\^<<gActor.theName>> pushes and twists {the dobj/he}, and it
        fall{s} over with a loud crash. ";
        if(contents.length > 0) {
          "As {the dobj/he} topples";
          foreach(local obj in contents){
           ", <<obj.theName>> falls off and plummets to the ground below";
           obj.moveIntoForTravel(loc);
          }
          ".\n";
        }  
        mainMoveInto(nil);
        moveInto(loc);
      }
    }
    dobjFor(Push) asDobjFor(Topple)
    dobjFor(Take) {
      check() {
        inherited;
        if(getWeight - weight > 0) {
          "{You/he} can\'t take {the dobj/him} while someone\'s on {it dobj/him}. ";
           exit;
        }
      }       
    }
    
;

class Wall: Fixture
    upperEndLoc = nil
    lowerEndLoc = (location)
    iobjFor(PutAgainst) {
        verify() { 
          if(!gActor.canTouch(self))
            illogical('{You/he} can\'t reach {the dobj/him} from here. ');  
        }
        action() { }
    }
    iobjFor(PutOn) asIobjFor(PutAgainst)
    curLadder = nil
    getCurLadder() { return curLadder; }
;

class DownWall : Fixture
  upperEndLoc = (location)
  lowerEndLoc = (lowerWall == nil ? nil : lowerWall.getOutermostRoom)
  lowerWall = nil
  iobjFor(DropDown)
  {
    verify() { 
          if(!gActor.canTouch(self))
            illogical('{You/he} can\'t reach {the dobj/him} from here. ');  
        }
    action() {}
  }
  iobjFor(PutOn) remapTo(DropDown, DirectObject, self)
;


DefineTIAction(DropDown)
;

VerbRule(DropDown)
  ('drop' | 'put' | 'lower' | 'lay') singleDobj 
    ('down' | 'onto' | 'on' 'to' | 'to' | 'down' 'to') singleIobj
  : DropDownAction
  verbPhrase = 'lower/lowering (what) (down what)'
;

DefineTAction(Topple)  
;

VerbRule(Topple)
  ('topple' singleDobj) | ('push' singleDobj 'over')
  :ToppleAction
  verbPhrase = 'topple/toppling (what)'
;


modify Thing
    dobjFor(PutAgainst) {
        preCond = [touchObj]
        verify() { illogical ('{You/he} can\'t do that. '); }
    }
    iobjFor(PutAgainst) {
        preCond = [touchObj]
        verify {
            illogical ('{You/he} can\'t put anything against that. ');
        }
    }
    dobjFor(DropDown){
       preCond = [touchObj]
       verify() { illogical ('{You/he} can\'t lower {that dobj/him}. '); }
    }
    iobjFor(DropDown) {
       preCond = [touchObj]
       verify() { illogical ('{You/he} can\'t lower anything down {that iobj/him}. '); }
    }
    dobjFor(Topple) {
       precCond = [touchObj]
       verify() { illogical ('{That dobj/he} {is} not something {you/he} can topple. '); }
    } 
;

DefineTIAction(PutAgainst)
;

VerbRule(PutAgainst)
    ('put' | 'lean' |'rest' | 'lay' ) singleDobj 'against' singleIobj
    : PutAgainstAction
    verbPhrase = 'put/putting (what) (against what)'
;
