#charset "us-ascii"
#include <adv3.h>
#include <en_us.h>

modify defaultSky 
  vocabWords = 'dark thunder sky/clouds/thunderclouds' 
  desc = "The sky is thick with dark thunderclouds, as if a storm is waiting
   to break in heaven as on earth. "
;

hill: OutdoorRoom 'Hillside'
    "To the south stretches a narrow valley, through which there winds a long
    road that leads up to the gate of a minor castle. This keep 
    is one of the many illegal fortifications that
    have sprung up in these lawless days of civil war. For a time it lay 
    abandoned, partly dismantled in half-hearted compliance with
    King Stephen's command, but the battle for
    Shrewsbury has made this area strategically important once more, so
    once again the castle is occupied and defended. "

    down = battlefield
    south asExit(down)
    north : NoTravelMessage { "The slope becomes too steep to climb that way. "; }
    up asExit(north)
    cannotGoThatWay() {
         reportFailure('You feel a strange compulsion to watch the battle. ');
         cannotGoShowExits(gActor);
    }
    actorInPrep = 'upon'
    leavingRoom(traveler) {
        setAnimateDaemons.execute();
    }
;

+Distant 'keep/stronghold/fort/fortress/fortifications/buildings/castle' 'keep'
    "It consists of four buildings, surrounding a central courtyard.
    The western face of the keep is walled off, with a gate meeting the
    road. "
;    
+Distant 'large larger small smaller battle/army/band/force/armies'
    'armies'
    "A small band, notionally loyal to the Empress Matilda,
      has taken the keep, and defends it against
    another, somewhat larger force -- King Stephen's troops no doubt. "
;
+Distant 'long road' 'road'
    "The road winds out of sight to the west, and terminates at the
    gate of the nearby keep. "
;

class Gate : Door

    checkDobjOpen {         
        reportFailure ('{You/he} cannot open {the dobj/him} from
                        here. '); 
        exit;
    }
    checkDobjClose {        
        reportFailure ('{You/he} cannot close {the dobj/him} from
                        here. '); 
        exit;
    }
;

road: OutdoorRoom 'Road to Main Gate'
    "The road rises here to a small castle strategically located at
    the centre of the valley.
    High walls and a wide gatehouse control access to the castle. The
    road winds eastward, down to a lower and wider part of the
    valley, while a long, narrow ditch runs along the south side
    of the road.\n
    The gate is currently
    <<mainGate.isOpen ? 'wide open' : 'shut tight'>>."
    west = mainGate
    north = battlefield
    east : FakeConnector {
                "You look as far as you can down the road, but there's
                no action to be seen in that direction. "
           }
    south = ditch
    actorInPrep = 'upon'
    name = 'road'
;

+ Enterable ->ditch 'long narrow ditch' 'ditch'
  "<<ditch.desc>>"
;

+ mainGate : Gate 'gate' 'gate'
;

gatehouse: Room 'Gatehouse'
    "This is a small, solidly built room, dominated by a large
    wheel. The wheel spins round a mast which stretches from the floor
    up to a great spindle running along the ceiling. Two thick ropes wind
    about the spindle, one on each end, and stretch down through the
    floor.\n
    To the north and south lie the tops of the keep walls. "
    north = northWall
    south = southWall
    name = 'gatehouse'
;

+wheel: Fixture 'mast/spindle/cog/wheel' 'wheel'
    "The wheel spins round a mast sunk in the floor and stretching to the
    ceiling. When the mast spins, the spindle going across the ceiling
    rotates, raising or lowering the ropes. (<<wheelRopes.desc>>) "
    dobjFor(Turn) {
        verify() { }
        check()
        {
          if(gActor.isGhost)
          {
            "There's no way {your/his} ghostly hands can get a purchase on
             that wheel - they simply pass through it. ";
             exit;
          }
        }
        action() {
            "{You/He} appl{ies} all {its actor/her} weight, and the wheel and
            mast slowly turn. The ropes
            <<mainGate.isOpen ? 'creak and twist about the spindle' :
            'slacken'>>. ";
            mainGate.makeOpen(!mainGate.isOpen);
        }
    }        
;

++ wheelRopes : Component 'ropes' 'ropes'
   "The ropes are currently <<mainGate.isOpen ? 'taughtly twisted about the 
    spindle' : 'loose and slack'>>. "
   isPlural = true
;

fieldWall : Wall '(north) (northern) (outside) castle/wall' 'castle wall'
        @battlefield
    upperEndLoc = northWall
;

northCourtWall : Wall '(north) (northern) (inside) castle/wall' 'northern wall'
        @courtyardNorth
    upperEndLoc = northWall
;


northWall: OutdoorRoom 'On the North Wall'
    "A wide stone wall, crumbling from recent neglect, and less
    recent battles, stretches
    from the steep hillside to the north, and ends in a gatehouse to the
    south. The battlements offer little in the way of
    coverage, and the alure is crumbling in places, providing only
    unsteady footing.\n
    From one side, the wall looks out across a narrow, densely wooded
    valley; the other affords a view of the grassy courtyard,
    surrounded by a scattering of buildings along the perimeter.
    At the farthest end of the courtyard stands the major structure
    of the stronghold: the keep containing the great hall and solar. 
    Its dominant frame rises two storeys high,
    and is capped by a broken tower; a balcony juts
    out from the solar. At the northern end of the courtyard a small, 
    slanted-roofed stable lies in disrepair.\n
    One may climb down a rickety staircase to the northern part of the
    courtyard<<
    seigeTower.distanceToWall == 0 ? ', or down the siege tower standing
    nearby' : '. A siege tower stands not far off'>> in the field below."

    
    locDesc = 'up on the northern wall'
    disambigName = 'northern wall'

    down : AskConnector {
        promptMessage = "From here you can reach either the battlefield or the
                        courtyard. "
        travelAction = ClimbDownAction
        travelObjs() {
            if (seigeTower.distanceToWall == 0)
                return ([toBattlefield, tocourtyardNorth]);
            return [tocourtyardNorth];
        }
        travelObjsPhrase = 'way'
    }
    actorInPrep = 'on'
    south = gatehouse
    name = 'north wall' 
;

+ DownWall 'wall down to field/battlefield/wall' 'battlefield'
  lowerWall = fieldWall
;

+ DownWall 'wall down to courtyard/wall' 'courtyard'
  lowerWall = northCourtWall
;

+toBattlefield : StairwayDown 'to the battlefield' 'to the battlefield'
    theName = 'to the battlefield'
    destination = battlefield
    canActorTravel(actor) { return (seigeTower.distanceToWall == 0); }
    describeDeparture(traveler, origin, dest) {}
    noteTraversal(traveler) {
        "{You/He} leap{s} upon the siege tower, and in a moment {you're}
        upon the battlefield. ";
    }
    cannotTravel() { "The siege tower is not close enough to the wall. "; }
    sightSize = small
;

+tocourtyardNorth : StairwayDown 'to the courtyard' 'to the courtyard'
    theName = 'to the courtyard'
    destination = courtyardNorth
    describeDeparture(traveler, origin, dest) {}
    noteTraversal(traveler) {
        "{You/He} carefully climb{s} down the rickety stairway to the
        courtyard.\n";
    }
;

DistanceConnector [northWall, battlefield, road];

DistanceConnector [northWall, courtyardNorth, balcony,
                   stableRoof, courtyardSouth, bellTower];

DistanceConnector [southWall, courtyardNorth, balcony,
                   kitchenRoof, courtyardSouth, bellTower];

DistanceConnector [southWall, ditch, road];

DistanceConnector [ditch, road, battlefield];

DistanceConnector [gatehouse, northWall];

DistanceConnector [gatehouse, southWall];

southCourtWall: Wall 'south southern (castle) wall' 'southern wall' @courtyardSouth
    upperEndLoc = southWall
;

southWall:  OutdoorRoom  'On the South Wall'
    "A wide stone wall, crumbling from over-hasty repair, stretches
    from the hillside to the south, and ends in a gatehouse to the
    north. The battered battlements offer little in the way of
    coverage.\n
    From one side, the wall overlooks a narrow, wooded valley, through which
    a road winds, flanked by a ditch running alongside. On the other side there 
    opens a grassed courtyard, surrounded by buildings along the inside perimeter.
    On the far end of the courtyard stands a chapel, opposite the central
    keep; from the chapel rises a short bell-tower. Closer, along the
    southern inside-wall, stands a building with a large chimney. "
    north = gatehouse
    south : NoTravelMessage { "The southern hillside is too steep to climb. " }
    locDesc = 'up on the southern wall'
    disambigName = 'southern wall'
    name = 'south wall'
    actorInPrep = 'on'
;

+ DownWall 'wall to ditch' 'ditch'
  lowerWall = ditchWall
  verifyDobjExamine { nonObvious; }  
;

+ DownWall 'wall to courtyard' 'courtyard'
  lowerWall = southCourtWall
;

ditchGround : defaultGround 'ankle-deep mud/ground' 'mud'
  "The mud underfoot is ankle-deep. "
  smellDesc = "The mud stinks of rotting refuse. "
;

ditch: OutdoorRoom 'In a ditch'
    "A metal grate in the keep's wall, by the southern cliff, dumps the
    keep's runoff and waste into this ditch. Though the runlet is dry,
    the ditch stinks with ankle-deep mud. The rutted dirt-road nearby 
    to the north runs parallel to the ditch from east to west, leading 
    up to the gated entrance of the solid, stone gatehouse. "
    north = road
    name = 'ditch'
    south : NoTravelMessage { "The hillside is too steep to climb. " }
    west : NoTravelMessage { "The castle wall is in the way. " }
    east : FakeConnector { "You start off down the ditch to the east,
      but some strange compulsion draws you back to the battle. " }      
    roomParts = [ditchGround, defaultSky] 
;

+ ditchWall : Wall 'south southern (castle) wall' 'castle wall'
    "This section of the castle wall is basically in good repair; it rises
     sharply out of the ditch, with a metal grating set near the bottom. "
    upperEndLoc = southWall
;

++ Component 'rusty metal iron grate/grating/bars' 'metal grate'
   "The grate is set into the castle wall at the northern end of the
    ditch. It's about eighteen inches square, and seems to be made
    of rusty iron. The bars of the grate are set too close together
    to crawl through, and it certainly doesn\'t look like something
    that could be pulled open. "
    cannotOpenMsg = 'The grate doesn\'t look like it was ever
     designed to be opened, but even if it was, it\'s now
     stuck fast. '
    dobjFor(Pull) asDobjFor(Open)
;


battlefield: OutdoorRoom 'Battlefield'
    "You stand in a field of tall grass and nettles. Large patches of
    the field are trodden black and muddied, signs of recent sorties
    engaged and broken, of a battle neither lost nor yet won.\n
    A siege tower stands here in the mud,
    <<seigeTower.distanceToWall == 0 ? 'close
    enough to the keep wall to climb onto it' : 'nearby to the
    keep\'s northern wall'>>.\n
    The steep hillside stretches up to the north, and to the south lies the
    road to the main gate. Beyond the road lies a ditch, and then the
    hills on the far side of the valley.\n
    <<battleDesc>> "
    up = seigeTower
    south = road
    west : NoTravelMessage { "The wall of the keep lies to the
                             immediate west. There is a siege
                             tower here, clearly designed to
                             gain entrance over the wall. " }
    locDesc = 'on the battlefield'
    north : FakeConnector {
                "There's no action to be found in that direction. "
            }
    east = north
    battleDesc() {
        foreach (local obj in contents) {
            if (obj.ofKind(Actor) && obj != gPlayerChar)
                return 'The sounds of battle crash in your ears. ';
        }
        return 'The sound of battle hangs in the air. ';
    }
    name = 'field'
;

+ Distant 'steep hillside' 'steep hillside'
  "The grassy hillside climbs gradually northwards at first, but then becomes
   too steep to climb, almost a sheer cliff blocking progress north. "
;

+seigeTower: Lever, StairwayUp 'to the wall siege tower' 'siege tower'
    "It's a rickety tower, made of a couple tree-trunks, and rolling
    on a creaking cart. <<distanceToWall == 0 ? 'It\'s near enough
    the wall that one could climb up and into the keep.' : 'It\'s
    too far away from the fortress wall to do much good at present.'>>"

    describeDeparture(traveler, origin, dest) {}
    noteTraversal(traveler) {
        "{You/He} climb{s} up the siege tower, and in a moment {you're}
        upon the north wall. ";
    }

    distanceToWall = 5
    isConnectorListed = (distanceToWall == 0)

    destination = northWall

    dobjFor(Push) {
        check() {          
          if(gActor.isGhost)
          {
            "{Your/his} body simply passes straight through it. ";
            exit;
          }
          inherited;
        }
        verify() {
            if (distanceToWall == 0)
                illogicalNow('It\'s already at the wall.');
        }
    }
    dobjFor(Pull) {
        check() {
          if(gActor.isGhost)
          {
            "{Your/his} ghostly hands make a feeble attempt to grasp
             hold of the tower, but simply pass through into the wood. ";
             exit;
          }
        }
        verify() {
            if (distanceToWall == 5)
                illogicalNow('It\'s already well away from the wall.');
        }
    }
    makePulled(pulled) {
        if (pulled) {
            if (distanceToWall < 5) {
                "{You/He} pull{s} the tower away from the wall. ";
                distanceToWall ++;
            }
        }
        else {
            if (distanceToWall > 0) {
                "{You/He} heave{s} with all {its/her} might, and
                inch{es} the siege tower towards the wall. ";
                distanceToWall --;
            }
        }
    }
    canActorTravel(actor) { return (seigeTower.distanceToWall == 0); }
    cannotTravel() { "The seige tower is not close enough to the wall. "; }
;

courtyardNorth: OutdoorRoom 'The northern courtyard'
    "The strong stench of horse emanates from the building to the
    north. It has not been long since the stalls were emptied -- the
    horses having fled or fallen. Nearby the stable wall a broken cart
    lies abandoned, and alongside it, a pile of worn-out and discarded
    horseshoes.\n    
    The ancient facade of the keep to the west is broken by a
    balcony on the middle storey. <<keepDoor.isOpen ? '\n
    A rickety wooden stairway struggles up the inside of the keep\'s
    outer wall. ' : nil>> "
    north = stable
    south = courtyardSouth
    east = mainGateInside     
    west = outsideKeepDoor
    up = northWall
    name = 'northern courtyard'
;

+ Decoration 'worn-out discarded pile*horseshoes' 'horseshoes'
  "The horseshoes are worn thin and broken, and thus of little use
   to man or horse. "
  isPlural = true
;

+ mainGateInside : Gate ->mainGate 'gate' 'gate'  
;

+wallStairs: StairwayUp 'to the northern north wall stairs' 'stairs'
    "A rickety wooden staircase struggles up the wall. "
    sightSize = small
    destination = northWall
    getDestination(origin, traveler) { return destination; }
;

+cart: Platform, Fixture 'cart' 'cart'
    "It's just an old, broken cart, with one wheel in pieces, and the
    other entirely missing. The low ceiling of the stable would be
    reachable from atop the cart. "
    up = stableRoof
    down = courtyardNorth
    dobjFor(Climb) asDobjFor(StandOn)
    dobjFor(ClimbUp) asDobjFor(StandOn)
;

+ Decoration 'broken (cart) wheel/cartwheel/hub/spokes' 'cartwheel'
  "The remaining cartwheel is little more than a hub with a trio of
  broken spokes. "
;

+ Fixture 'stable roof' 'stable roof'
    "The stable roof overhangs the courtyard by several feet. "
    iobjFor(PutAgainst) {
        verify() {}
        check() {
            "The roof of the stable is at too great an angle. On the
            other hand, the nearby cart appears climbable. ";
            exit;   
        }  
    }
;

+ balconyWall : Wall 'balcony' 'balcony'
    "The balcony juts out from the side of the keep. "
    upperEndLoc = balcony
;

flagpole: MultiLoc, Fixture 'flagpole/pole' 'flagpole'
    "It's a tall and sturdy flagpole. "
    locationList = [courtyardSouth, courtyardNorth]
    specialDesc = "A tall flagpole stands in the middle of the courtyard. "    
;

rope: Thing 'rope' 'rope'
    "It's a rope attached to the flagpole. "
    location = flagpole
    specialDesc = "A rope is attached to the flagpole. "
    remoteSpecialDesc(pov) {
      if(pov.canTouch(self))
        specialDesc;
    }
    dobjFor(SwingOn) {
        precond = [objHeld] // why doesn't this work by itself?
        verify() {
            if (rope.location != gActor) // why is this necessary?
                illogicalNow('{You/he} must be holding the rope. ');
        }
        check() { }
        action() {
            if (gActor.location is in (courtyardNorth, courtyardSouth)) {
                "{You/He} swing{s} on the rope a bit, but {are}
                unable to get up much momentum. ";
                return;
            }
            if (gActor.location is in (northWall, balcony)) {
                "{You/He} swing{s} across the courtyard, and up
                onto <<gActor.location == northWall ? 'the balcony of the
                keep' : 'the northern wall of the keep'>>.\n";
                gActor.mainMoveInto((gActor.location == northWall ?
                                          balcony : northWall));
                if (gActor == gPlayerChar)
                    gActor.lookAround(gameMain.verboseMode);
                return;
            }
            if (gActor.location is in (kitchenRoof, stableRoof)) {
                "{You/He} swing{s} across the courtyard, and up
                onto <<gActor.location == kitchenRoof ? 'the roof of the stables' :
                'the roof of the kitchen'>>.\n";
                gActor.mainMoveInto((gActor.location == kitchenRoof ?
                                          stableRoof : kitchenRoof));
                if (gActor == gPlayerChar)
                    gActor.lookAround(gameMain.verboseMode);
                return;
            }
            else {
                "{You/He} swing{s} into the courtyard. ";
                gActor.mainMoveInto(courtyardNorth);
                if (gActor == gPlayerChar)
                    gActor.lookAround(gameMain.verboseMode);
                return;
            }
        }
    }

    afterAction() {
        if (location != flagpole &&
            getOutermostRoom not in (courtyardNorth, courtyardSouth,
                     northWall, balcony, kitchenRoof, stableRoof)) {
            "The rope snaps back into the courtyard.\n";
            mainMoveInto(flagpole);
        }
    }
;

stable: Room 'Stable'
    "The ramshackle stable-block stands empty. Columns of light shine
    through holes in the roof of the otherwise dingy stalls. Stray
    wisps of hay hang from the deserted mangers, the stone water trough
    smells dank and foul, and the tackle hanging from a row of pegs
    gleams dully, its fading lustre awaiting the attention of the
    long-fled stable-hand.\n
    The southern door to the stables has fallen off its rusted hinges,
    allowing one to see straight out to the surrounding courtyard. "
    south = courtyardNorth
    out asExit(south)
    name = 'stable'
;

DistanceConnector [stable, courtyardNorth];

/* write a beforeAction for dropping the actor through the stable roof
 * into the stable, if the actor is fighting. Maybe a 20% chance.
 */

stableRoof: OutdoorRoom 'Roof of the Stable'
    "The old stable roof shows signs of disrepair -- ancient, buckled
    joists jut through cracked slates, and the frequent holes are
    patched only by spreading lichen.\n
    The keep's balcony is tantalisingly out of reach. Father off, 
    across the courtyard, lies another building to the south, and in
    the opposite corner stands an ageing chapel. "
    down : OneWayRoomConnector {
           ->cart
           noteTraversal(actor) { "{You/he} climb{s} down onto the cart.\n"; }
           } 
    actorInPrep = 'on'
;

courtyardSouth: OutdoorRoom 'Southern Courtyard'
    "No smoke rises from the kitchen to the south; the keep's need for
    defence has for the moment overshadowed the simpler needs of its
    defenders. Pots are piled beside the entrance.\n
    The courtyard stretches out to the north, and the small chapel, to
    the west, bears no evidence of recent use, though the bell
    tower would make a fine roost for an archer. "
    north = courtyardNorth
    west = chapel
    south = kitchen
    name = 'southern courtyard'
;

+ Decoration 'pot/pots' 'pots'
    "There are a few pots, piled causally. "
    isPlural = true
;

+ Wall 'kitchen roof' 'kitchen roof'
    upperEndLoc = kitchenRoof
;

kitchen: Room 'Kitchen'
    "Right now, this kitchen is little more than a shelter
    from arrows. Light shines through the front door to the courtyard,
    and a shread of light falls on the dirty embers of the fireplace.
    <<!cauldron.moved ? 'The huge iron cauldron lies tipped on its side,
    and moths' : 'Moths'>> flutter
    around the uncovered flour barrel. <<kitchenTable.isIn(self) ?
    'The stolid kitchen table stands bare of utensils, its wooden knife 
     rack emptied, as every potential
    weapon has been pressed into use for the ongoing fray. ' : nil >>"
    north = courtyardSouth
    south = chimney
    out asExit(north)
    name = 'kitchen'
;

+ cauldron : Attachable, Container, Heavy 'huge iron cauldron/cauldron' 'cauldron'
    desc()
    {
      if(isAttachedTo(kitchenTable))
        "It's wedged firmly to the end of the kitchen table, forming a hard
         iron cap. ";
      else
        "It's large enough to cook a fair-sized sheep in. ";
    }
    initDesc = "It lies on its side, near the fireplace. "
    canAttachTo(obj) { return obj==kitchenTable; }
    handleAttach(obj)
    {
      "{You/he} wedge{s} the iron cauldron on the end of the sturdy kitchen table. ";
    }
    moveWhileAttached(movedObj, newCont)
     {
        if(movedObj == self)
        {
          "{You/he} pull{s} the cauldron free of the kitchen table. ";
          detachFrom(kitchenTable);
        }
        else
        {
          baseMoveInto(newCont);
          moved = true;
        }
     }

;

+Decoration 'flour barrel' 'flour barrel'
    "It's mostly empty, and what remains has been soured by moisture. "
;

+ kitchenTable : Attachable, ComplexContainer, TravelPushable '(kitchen) table' 'kitchen table'
  "It's battered, but basically solid and sturdy. "
  subSurface : ComplexComponent, Surface { }
  subUnderside : ComplexComponent, Underside { }
  isMajorItemFor(obj) { return obj==cauldron; }
  useSpecialDesc = (!isIn(kitchen))
  specialDesc = "A large wooden table rests on the ground. "
  describeMovePushable(traveler, connector)
  {
    "<.p>{You/he} push{es} the heavy table into <<location.destName>>. ";
  }
  tryBattering(connector) 
  { 
    if(connector == outsideKeepDoor && isAttachedTo(cauldron) && keepDoor.isLocked)
      battering.doScript;
  }
  battering : EventList {
    [
      'The cauldron is pressed hard against the keep door. ',
      
      '{You/he} move{s} the table back and then batter{s} it against the door. ',
      
      'There\'s a sound of splintering as the makeshift battering ram strikes the door. ',
      
      'The door starts to give it as the cauldron-headed table-ram strikes it. ',
      
      new function { 
        "With a loud splintering sound, the door gives way. ";
        keepDoor.makeLocked(nil);
        keepDoor.makeOpen(nil);
      }
    ]
  } 
  dobjFor(PushTravel)
  {
    check() {
      if(gActor.isGhost)
      {
        "{Your/his} ghostly frame passes straight through the table
         without budging it an inch. ";
         exit;
      }
    }
    action() {
      local conn;
      if(gActionIs(PushTravelThrough))
        conn = gAction.iobjCur_;
      if(gActionIs(PushTravelDir))
      {
        local dir = gAction.getDirection;
        conn = gActor.location.getTravelConnector(dir, gActor);
      }
      if(conn == outsideKeepDoor) tryBattering(conn);
      inherited;
    }
  }
;


++ Decoration 'wooden knife rack' 'knife rack'
  subLocation = &subSurface
;

++ cheese : Food 'lump/cheese cheddar' 'lump of cheese'
  "It's a large wedge of cheddar - extra-strong by the smell of it. "
  subLocation = &subSurface
  smellDesc = "It smells pretty ripe. "
  tasteDesc = "It's obviously a <i>very</i> mature cheddar. "
;

DistanceConnector [kitchen, chimney];

chimney: Room 'Chimney'
    "It's nasty sooty in here. There's a ladder built into the side
    of the chimney; the stormy sky shines down from above. "
    north = kitchen
    up = chimneyLadderUp
    canTravelerPass (traveler) {
        foreach (local obj in traveler.contents) {
            if (obj.ofKind(Ladder))
                return nil;
        }
        return true;
    } 
    explainTravelBarrier (traveler) {
        "\^<<traveler.theName>> would be hard pressed to carry a ladder
        into the chimney." ; 
    }
    name = 'chimney'
;

+ chimneyLadderUp : StairwayUp 'rotten charred wooden ladder' 'ladder'
  "The wooden ladder leading up the inside of the chimney looks
   charred and rotten in places (not surprising, given that it's a 
   wooden ladder in a chimney!), but it may be climbable. "   
;

kitchenRoof: OutdoorRoom 'Kitchen Roof'
    "A flat, slate-shingled roof with a crooked chimney -- for the
    moment, however, no smoke rises from the cold chimney's sooted
    cowl.\n
    The courtyard below stretches out to the north. Across the way
    stands another building, and westward lies a chapel. On the
    opposite corner of the courtyard stands the central keep. "
    down = roofChimney
    name = 'kitchen roof'
    actorInPrep = 'on'
;

+ roofChimney : StairwayDown ->chimneyLadderUp 'battered sooty sooted chimney/ladder/cowl' 
    'chimney'
   "The battered chimney is capped with a sooty cowl, but there's enough
    space to crawl under the cowl and climb down the ladder that runs down
    inside the chimney. "
;

chapel: Room 'Chapel'
    "The chisled saints look on stonily from their alcoves as the pious
    tranquility of the decrepit chapel is shattered by the raucous
    clamour of battle. The rafters hang in shadow, and from this
    darkness spirals down some worm-eaten stairs, the bottom of which
    have rotted to dust. The intricate carvings on the
    stone pulpit reflect the play of light and shadow cast from the
    small, heavily leaded windows.\n
    <<bible.moved ? '' : 'The weighty Bible lies open yet upon the lectern, 
    its satin marker aiming hopelessly to preserve some lesson from yester-year; the dust
    has since then fallen thick, and such matters are quickly forgot
    in times of more pressing needs.'>> Following the line of the nave
    eastwards, one may pass through the columnar arcades of the atrium
    and back out, blinking, into the bright sunlight of the open
    courtyard. "
    east = courtyardSouth
    name = 'chapel'
    up : OneWayRoomConnector {
      destination = (myLadder==nil ? brokenChapelStairs : myLadder)        
      myLadder = (chapelWall.getCurLadder)
      isConnectorApparent (origin, actor) { return actor.isGhost || myLadder != nil; }
      canTravelerPass (traveler) { return traveler.isGhost || myLadder != nil; }
      explainTravelBarrier(traveler) {
        "{You/he} can't reach the bottom of the broken stairway from the
         ground. ";
      }
      noteTraversal(traveler) {
       if(traveler == gPlayerChar && myLadder ==nil)
         "Although a living human being would be unable to reach the bottom
         of the broken stairway from here, somehow {your} ghostly body
         translates {your} wish into action, so that {your} phantom frame
         floats up to the foot of the broken stairs. ";
      }
    }
;


+ chapelWall : Wall 'broken collapsed tower stairs/stairway/staircase' 'broken stairs'
    upperEndLoc = brokenChapelStairs
;

+ Surface, Heavy 'heavy brass heraldic stand/lectern/eagle/bookrest' 'lectern'
  "It's a heavy brass stand, with a bookrest in the form of a huge heraldic
    eagle. "
;

++ bible : Readable 'weighty bible' 'bible'
  "The bible is bound in black leather and lovingly written in a monkish hand. "
  readDesc = "It's is open at a passage which reads:\n
  Hiericho autem clausa erat atque munita timore filiorum Israhel et 
  nullus egredi audebat aut ingredi dixitque Dominus ad Iosue: ecce dedi in manus tuas 
  Hiericho et regem eius omnesque fortes viros circuite urbem cuncti bellatores 
  semel per diem sic facietis sex diebus septimo autem die sacerdotes tollant 
  septem bucinas quarum usus est in iobeleo et praecedant arcam foederis septiesque 
  circuibitis civitatem et sacerdotes clangent bucinis cumque insonuerit vox 
  tubae longior atque concisior et in auribus vestris increpuerit conclamabit omnis 
  populus vociferatione maxima et muri funditus corruent civitatis ingredienturque 
  singuli per locum contra quem steterint ...\b
  Too bad you never learned to read Latin; but then since you're only an 
  uneducated Saxon peasant you never learned to read at all, and death seems
  not to have made up for that lack of schooling in life. " 
  initSpecialDesc = ""
;

+ Booth, Fixture 'stone intricate pulpit/carvings' 'pulpit'
  "The stone pulpit stands to the north of the altar. The intricate
   carvings on the former are mainly of vines and grapes, but a closer
   examination does reveal a carved crucifix. " 
;

++ Component 'carved jesus christ/crucifix/cross' 'crucifix'
  "The crucifix carved into the side of the pulpit depicts Christ on
   the cross - a sermon in stone. "
;

+ Surface, Fixture 'stone altar' 'altar'
  "The stone altar is bare, with neither altar-cloth, cross nor candle 
  upon it, perhaps a sign that mass has not been said here for quite
  a while. "
;

brokenChapelStairs: Room 'On a Crumbling Stairway' 
    "You stand near the broken end of the bell-tower stairway, which
    spirals up into darkness. A dangerous drop would end on the bare
    and dusty floor of the chapel. "
    // dropDestination = chapel
    down : OneWayRoomConnector {
      destination = (myLadder==nil ? chapel : myLadder)        
      myLadder = (chapelWall.getCurLadder)
      isConnectorApparent (origin, actor) { return actor.isGhost || myLadder != nil; }
      canTravelerPass (traveler) { return traveler.isGhost || myLadder != nil; }
      explainTravelBarrier(traveler) {
        "It's too far down to the ground to jump down from here. ";
      }
      noteTraversal(traveler) {
       if(traveler == gPlayerChar && myLadder ==nil)
         "Although a living person would be unable to jump down to the
         chapel floor from here without grave risk of injury, one
         advantage of being dead is that {you/he} {is} immune to all
         further injury, so {you/he} leap{s} off the bottom step and
         waft{s} safely down to the chapel floor.<.p>";
      }
    }
    up = chapelStairsUp
    name = 'crumbling stairway'
;

+ DownWall 'dusty chapel floor/chapel' 'chapel floor'
  lowerWall = chapelWall
;

+ chapelStairsUp : StairwayUp 'spiral stairs stairway' 'spiral stairway'
  "The spiral staircase leads up into darkness; although the lower section
   of the staircase has rotted away, the section leading upwards looks
   reasonaby secure. "
;

DistanceConnector [chapel, brokenChapelStairs];

bellTower: Room 'Bell Tower'
    "Cloistered high in the churchtower that dominates the main keep,
    the ponderous old warning bell hangs eerily silent.  
    Fallen into the same sad repair as
    the tower in which it is housed, the mighty bell hangs lame, its
    hammer fallen.\n
    From so high a perch one commands nearly a \"God's eye-view\" view
    of the environs: the main keep with its jutting balcony is open
    clear; the stable roof, at the cross corner of the keep, appears
    in miniature, scaled-down to a house for pocket-size horses,
    with the diminutive kitchen block standing across-side. Each
    doll-sized feature is set down upon a grassy courtyard that, from
    this vantage, appears no larger than a child's blanket. "
    down = brokenChapelStairsDown
    name = 'bell tower'
;

+ brokenChapelStairsDown : StairwayDown ->chapelStairsUp
  'spiral stairs/stairway/staircase' 'spiral staircase'
  "The staircase spirals down the inside of the tower. "
;

+ Decoration 'ponderous old warning mighty bell/beam/crack' 'bell'
  "It's a large bell, hanging from a beam, but a close inspection reveals
   a crack in its side; it also lacks its clapper.\n
   Its broad clamour should, by rights, peal out across the countryside, calling
    all and sundry to battle, but at this hour, its mute silence hangs
   in the air, an ominous portent."
;

keepDoor: Lockable, Door 'great main keep door' 'keep door'
    "Though the keep is in disrepair, this door is quite strong and
    most likely impregnible. "
    location = keep
    initiallyLocked = nil
    initiallyOpen = true
;

outsideKeepDoor: LockableWithKey, Door ->keepDoor 'great main keep door' 'keep door'
   "Though the keep is in disrepair, this door has been recently replaced
    and looks remarkably robust. "
    location = courtyardNorth      
;

keep: Room 'Great Hall'
    "Once the scene of many a lavish banquet, now the great hall echoes
    no longer with laughter and feasting, and suggests but the memories of 
    chivalric cheer and camradary. The massive wrought-iron roasting spit stands
    empty in the gaping hearth; in the minstrel's gallery a
    solitary, broken lute lies abandoned.\n
    The sturdy oaken table that once groaned under the weight of its 
    sumptuous spread is now covered only by a thickening layer of 
    dust, and the flanking benches no longer seat the merry
    throng of knights, noblemen, squires and their good ladies. The
    great stairway leading to the solar above was once a-swish 
    with the rustling trains of the fair, or the impatient clatter of
    armoured knights with minds set far away on distant quests.\n
    Of late, however, few have trodden its path, nor ventured out to
    the eastern courtyard to catch the cool evening breeze and escape
    the heated atmosphere of the festivities. "
    east = keepDoor
    out asExit(east)
    up = keepStairway
;

+ hallTable : ComplexContainer, Platform, Heavy 'sturdy oaken oak table' 'sturdy oaken table'
  "The oak table is about ten feet long by four wide, and is covered with
   thick dust. "
   subSurface : ComplexComponent, Surface { }
   subUnderside : ComplexComponent, Underside { }
;

++ Decoration 'dust' 'dust'
  "Thick dust covers the table. "
;

class Bench : Chair, Heavy 'bench*benches' 'bench'
   "It's a long wooden bench. "
   isEquivalent = true
   bulkCapacity = 50
;

+ Bench;
+ Bench;

+ keepStairway : StairwayUp 'stone stairway/stair/stairs' 'stairway'
    "The stone stairway leads upwards. "
    destination = chamber
;

+ Container, Fixture 'hearth' 'hearth'
  "The massive stone hearth contains nothing but the blackened ash of
   some long-dead fire, and a bare iron roasting spit. "

;

++ Component 'massive wrought iron roasting spit' 'roasting spit'
  "The iron spit looks massive enough to roast a whole boar at once. "
;

+ HighNestedRoom, Booth, Fixture 'minstrels\' minstrels gallery' 'minstrels\' gallery'
  "The minstrel's gallery is a wooden plaftform high up on the southern wall. "
  stagingLocations = [hallTable]
  exitDestination = hallTable
  isStagingLocationKnown(loc) { return nil; }
;

++ Decoration 'broken lute' 'broken lute'
  "The broken lute lies on the floor of the minstrels' gallery, a crack in its
   side and only one string left unbroken. It's obviously unplayable. "
;

chamber: Room 'Solar'
    "Torn and faded tapestries hang from the walls, lone testimony to
    the keep's former grandeur. Perhaps this was once a lavish
    repository for the spoils of battle: the fruits of the first
    crusade spilling from drawers, chests full of gold, jewels, rich
    silks from the Orient piled indolently, flashing their vibrant
    colours at the sombre oak panneling -- all have long since
    vanished. Today, the only flash of colour to lighten the settling
    drab creeps through the open doorway, -- a grey-pink haze of
    a stormy dawn floods the room, picking out the relief on the
    balcony railings.\n
    It is not difficult to imagine the keep at the height of its
    splendour; the now beaten stone stairway ringing with the
    insistent tread of servants and gentlefolk as they hurried
    downstairs, contrasting with the creak and groan of footfall on the
    worn wooden stairs that wind reluctantly up towards the tower. "
    
    east = balcony
    up = chamberStair
    down = keep
    name = 'solar'
;

+Decoration 'torn faded tapestry/tapestries' 'tapestries'
    "They commemorate some long-forgotten battle; the valiant knights
    depicted by the fraying needle-work have long since laid down their
    weapons, and the nimble fingers that wove the ancient hanging
    have too passed beyond living memory. "
;

+ chamberStair : StairwayUp 'stair/stairs/stairway' 'stair'
    "The stair winds upwards into a tower. "  
;

DistanceConnector [chamber, balcony];

balcony: OutdoorRoom 'On a balcony'
    "You are standing on the balcony of the main keep. From this
    vantage, you can see the entire courtyard, the walls surrounding
    the fortress, and the kitchen and stable, and the chapel
    bell-tower. "
    west = chamber
    name = 'balcony'
    actorInPrep = 'on'
;
 
+ DownWall 'courtyard' 'courtyard'
  lowerWall = 'balconyWall'
;

tower: Room 'Top of the Tower'
    "The narrowing staircase ends in a dingy, cramped room, half
    collapsed. Light enters this windowless room from holes in the
    roof, and there is a persistent musty odour of damp and decay.
    Exposure to the elements has left the rough-hewn floorboards
    buckled and stained. "
    down = towerStair
;

+ towerStair : StairwayDown ->chamberStair 'stair/stairs/stairway'
                                           'stairway'
    "The stairway winds down the tower. "
;

ladder3: Ladder 'short ladder' 'short ladder' @chapel
   "It's a short wooden ladder. "
   initSpecialDesc = "A short wooden ladder lies on the ground near the pulpit. "
   useSpecialDesc { return (!moved || inherited); }
;

gameMain: GameMainDef
    initialPlayerChar = ghost
    showIntro()
    {
   "<.p>One could only look dispassionately upon the warriors below.
   Clumsy and crude, they are, and irrelevant. Stephen's a weakling and
   Matilda's a fool, as the past many years have more than adequately shown.
   They say that Christ and his saints are sleeping -- but wait there's something
   near.\b

  -- \"There's one!\"\b

  -- A bow snaps.\b

  ** You have died. **\b";
  morePrompt();


showTitle();
showDeathImage();
    }
    
    showGoodbye()
    {
      "<.p>Thanks for playing!\b";
    }
    showTitle() {
        "<b><<versionInfo.name>></b>\n
        <<versionInfo.byLine>>\b";
    }
    showDeathImage() {
       "
--Ah, just a hunter, it looks like.\n
--Nah. Well. Best to be safe.\n
--Well, anyway, no ambush. Let's head back.\b

One of the younger scouts looks compassionately at your corpse, but
turns quickly back to the scouting party, and is in a moment gone down
the hill.\b
They say that Christ and his Saints are sleeping, and so, it seems, are you.
       Your poor, bloodied Saxon body lies at your ghostly feet, while below
       rages the battle that cost you your life. Not that you care which member of
       the conquering family ultimately comes out on top; from all accounts Stephen
       is a weak fool and Matilda a haughty one, and both alike are Normans. Yet 
       somehow you feel compelled to watch the outcome of this battle before you
       go to your final rest, if only the better to understand what you died for.<.p>";
       myCorpse.baseMoveInto(gPlayerChar.location);
    }
;

versionInfo: GameID
   name = 'The Battle of Walcot Keep'
   showAbout() {
     "This game was written for the 2004 IF-Art Competition,
      and hopes to open and explore a certain kind of observation-based
      interactivity. It was also envisioned as a demonstration and test
      of the RAP engine.\b

      As you have already gathered, the character you control is a
      ghost. As such, the player-character cannot affect anything: you are
      involved in the game simply as an observer and explorer. The
      action of the game is randomized and somewhat chaotic, and this \"play\"
      varies outcome of the game and the choice of strategies of
      the automatons (soldiers) in different situations. But the
      prime variation is determined by <i>your</i> choice and timing of moves.
      (To frame this project,
      one might say it is an exploration of this relation of automation
      and perspective, the interactive character of active inspection and observation of a
      multi-dimensional event, etc. -- but hopefully it's fun too: enjoy!)\b

      For further information type <A HREF='HELP'>HELP</A> ";
   }
   byLine = 'By Steve Breslin, Eric Eve, and Lindsey Hair\nIllustrated by Michael Bechard'
    htmlByline =
        'by <a href="mailto:versim@hotmail.com">Steve Breslin</a>, Eric Eve, and
         Lindsey Hair\nIllustrated by Michael Bechard'
   version = '1.0'
   showCredit() {
       "<b><<name>></b>\n
       <<byLine>>\b
       Thanks to everyone who helped with the development of RAP, especially
       Nate Cull, who invented it, and thanks as well to everyone who helped
       with Tads-3 Combat. Thanks to everyone who helped with the architecture
       and listing mechanisms. Thanks to the v-space group. Thanks to Andreas
       Sewe for playtesting. And thank you vere much for playing; we can only
       hope you're enjoying the game as much as we enjoyed writing it.\b";
   }
   authorEmail = 'Steve Breslin <versim@hotmail.com>'

;

modify statusLine
    showStatusHtml()
    {
        /* hyperlink the location name to a "look around" command */
        "<a plain href='<<libMessages.commandLookAround>>'>";
            
        /* show the left part of the status line */
        showStatusLeft();
            
        /* set up for the score part on the right half */
        "</a><tab align=right><a
            href='show map'>";
        
        /* show the right part of the status line */
        showStatusRight();
        
        /* end the score link */
        "</a>";
        
        /* add the status-line exit list, if desired */
        if (gPlayerChar.location != nil)
            gPlayerChar.location.showStatuslineExits();
    }

    showStatusRight() {
        "Display map";
    }
;
