#charset "us-ascii"
/*

   == Tads-3 Combat RAP Planbase Extension

   == Copyright (c) 2004, Steve Breslin
 
   ==>>  License:
     _______________________________________________________________
    /                                                               \
    |     You can use this material however you want, but if you    |
    |     decide to distribute anything that uses, #include's or    |
    |     otherwise borrows from this program, you have to send     |
    |     a copy of any improvements or modify-cations you made     |
    |     to this program to its author, Steve Breslin.             |
    |                                                               |
    |     That way, you will help keep this material up to date for |
    |     everybody else, and everybody else will help keep it      |
    |     up to date for you.                                       |
    |                                                               |
    |     You may redistribute this verbatim or in modified form    |
    |     only if you keep the copyright and license intact.        |
    |                                                               |
    |     Also, feel encouraged to release your source code along   |
    |     with your game, though this isn't a requirement.          |
    |                                                               |
    |     The author can be reached at <versim@hotmail.com>.        |
    \______________________________________________________________*/

/* This file is only for use with RAP -- Reactive Agent Planner.
 *
 * It enables you to direct combat-actors to seek and attack
 * other actors.
 */

#include <adv3.h>
#include <en_us.h>

/* This is a one-time attack. If combat is not automated, the actor
 * will attack the target once, and then just sit there.
 */
rAttack: rStep
    rTrue(a, p) { return nil; }
    rPlans(a, p) { return ( [ [rBe, rAttackable, p,
                               rDo, rAttack, p] ] );
    }
    rAction(a, p) {
        nestedActorAction(a, Attack, p);
    }
;

/* This is more thoroughgoing than rAttack, because the actor will
 * continue to hunt until the target is dead.
 */
rKill: rStep
    rTrue(a, p) { return (!p || !p.location); }
    rPlans(a, p) { return ( [ [rBe, rAttackable, p,
                               rDo, rAttack, p] ] );
    }
;

rAutomateCombat: rStep

// for this rTrue definition, try:
// if (!a.myCombatTarget) {
//     rapAct(rFindAndAttack, nil);
//     return true;
// }
// return nil;
// it returns true after the new rap action goes through.

    rTrue(a, p) { return (a.myCombatTarget.location); }
    rPlans(a, p) {
        if (a.myCombatTarget)
            return ( [[rBe, rAttack, a.myCombatTarget]] );
        return [];
    }
;

rFindAndAttack: rStep
    rTrue(a, p) { return nil; }
    rPlans(a, p) {
        if (!p.isClass) {
            return( [[ rBe, rAttack, p ]] );
        }
        else {

// This technique is probably too slow:

/*
            foreach(local obj in a.scopeList)
                if (obj.ofKind(p) && a.canAttack(obj))
                    return ([[ rDo, rAttack, obj ]]);
*/

// This seems faster:

            local ret = [];
            for(local obj = firstObj(p) ; obj ; obj = nextObj(obj, p)) {
                ret += [ [ rBe, rAttack, obj ] ] ;
            }
        return ret;
        }
    }
;

/*
 * This could probably be improved upon, but the current plan to make
 * a target attackable is to move "reach-wise" towards the target,
 * until the target becomes attackable. This may mean an archer moves
 * away from a roost, towards a dragon, so not very smart.
 */
rAttackable: rStep
    rTrue(a, p) {
        if (!a.wielding)
            a.autoWield();
        return (a.canAttack(p));
    }
    rPlans(a, p) {
        return (rReachable.rPlans(a, p));
    }
;

/* This is a directive for the NPC to wait in his current room, but be
 * in aggressive mode when anyone enters scope.
 */
rAmbush: rStep
    rTrue(a, p) {
        if (!a.wielding)
            a.autoWield();
        return nil;
    }
    rPlans(a, p) {
        if (!p.isClass) {
            if (a.canAttack(p))
                return( [ [ rDo, rAttack, p ] ] );
        }
        else {

// This is problematically slow:

/*
            foreach(local obj in a.scopeList)
                if (obj.ofKind(p) && a.canAttack(obj))
                    return ( [[ rDo, rAttack, obj ]] );
*/

// This works ok:

            for (local obj = firstObj(p) ; obj ; obj = nextObj(obj, p))
                if (a.canAttack(obj))
                    return ( [[ rDo, rAttack, obj ]] );
        }
        return ([]);
    }        
;

